@@ -8,6 +8,7 @@
 # @Software     : PyCharm
 # @Description  : 
 
+from io import BytesIO
 from meutils.pipe import *
 
 from starlette.status import *
@@ -78,6 +79,12 @@ class App(object):
         handler = self._handler(func, method, **kwargs)
         self.app.api_route(path=path, methods=[method])(handler)
 
+    def add_route_plus(self, path='/xxx', func=lambda x='demo': x, method="GET", **kwargs):
+
+        handler = self._handler_plus(func, **kwargs)
+
+        self.app.api_route(path=path, methods=[method])(handler)  # method
+
     def add_apps(self, app_dir='apps', main_func='main', **kwargs):
         """加载当前app_dir文件夹下的所有app（递归）, 入口函数都是main
         1. 过滤掉 _ 开头的py文件
@@ -125,45 +132,44 @@ class App(object):
                 return self._try_func(input, func, result_key, **kwargs)
 
         elif method == 'POST':
-            async def handler(kwargs_: dict):  # todo 表单 request.form()
+            async def handler(kwargs_: dict):
                 input = kwargs_
                 return self._try_func(input, func, result_key, **kwargs)
 
-        # todo: def get_and_post(request: Request, kwargs = Body(None)):
         else:
             async def handler():
                 return {'Warning': 'method not in {"GET", "POST"}'}
 
         return handler
 
-    def _handler_new(self, func, method='GET', result_key='data', **kwargs):
+    def _handler_plus(self, func, result_key='data', **kwargs):
         """
 
         :param func:
-        :param method:
-            get -> request: Request
-            post -> kwargs: dict
         :param result_key:
         :return:
         """
 
-        if method == 'GET':
-            async def handler(request: Request):
-                input = request.query_params._dict
-                return self._try_func(input, func, result_key, **kwargs)
+        async def handler(request: Request, file: UploadFile = File(...)):  # todo: 兼容其他请求，比如 request.form()
+            body_input = await request.json()
+            io = BytesIO(await file.read())  # 字节型
+            query_input = request.query_params._dict
+            input = {**query_input, **body_input}
 
-        elif method == 'POST':
-            async def handler(kwargs_: dict):  # todo 表单 request.form()
-                input = kwargs_
-                return self._try_func(input, func, result_key, **kwargs)
+            if io:
+                input['__io'] = io
 
-        # todo: def get_and_post(request: Request):
-        # query_input = request.query_params._dict
-        # body_input = await request.json()
+            logger.info(f"io: {io}")
+            logger.info(f"query_input: {query_input}")
+            logger.info(f"body_input: {body_input}")
+            logger.info(f"input: {input}")
 
-        else:
-            async def handler():
-                return {'Warning': 'method not in {"GET", "POST"}'}
+            duplicate_key = query_input.keys() & body_input.keys()
+
+            if duplicate_key:
+                logger.warning(f"Duplicate key:{duplicate_key}")
+
+            return self._try_func(input, func, result_key, **kwargs)
 
         return handler
 
